/*

                                   _JNJ`                        
                               .JNMH`                           
                              JMMF`       `;.                   
                            .NMM)          `MN.                 
                            MMM)            (MML                
                           (MMM`             MMML               
                     M  I  D  N  I  G  H  T    C  o  D  E       
                           (NMMF             MHNH               
                            NMML            .MMM                
                             NMML          .NMH                 
                              4MMNL       .#F                   
                               `4HNNL_   `                      
                                   `"""`                        

                          Copyright (C) 2004-2014
       "Ian (Larry) Latter" <ian dot latter at midnightcode dot org>

    Midnight Code is a registered trademark of Ian Latter.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, and mirrored at the Midnight Code web
    site; as at version 2 of the License only.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    (version 2) along with this program; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA, or see http://midnightcode.org/gplv2.txt

*/

#include <SoftwareSerial.h>


// User Preferences
int with_light = true;
int with_sound = true;
int start_delay = 5;    // x 1 second
int timeout = 10;       // x 10 ms
int debug = false;
int ack_delay = 1;      // x 1 ms
int end_on_timeout = false;

// Hardware Config
int led = 13;
int piezo = 11;
int PCpin_rxd = 8;
int PCpin_txd = 9;

// Operational Globals
int counter = -1;
unsigned int offset = 0;
unsigned int mode = 99;  // force initial dispatch of data mode
unsigned int processed_octets = 0;
byte keys[6];

SoftwareSerial PCserial(PCpin_rxd, PCpin_txd);


void 
piezo_beep() {
  int frequency = 4000;
  
  for(int i = 0; i < ((frequency / 1000) * 400); i++) {
    digitalWrite(piezo, HIGH);
    delayMicroseconds(500000 / frequency);
    digitalWrite(piezo, LOW);
    delayMicroseconds(500000 / frequency);
  }
  
}


void 
user_notify(unsigned int count) {
  
  for(int i = 0; i < count; i++) {
    if(with_light) {
      digitalWrite(led, HIGH);
    }
    if(with_sound) {
      piezo_beep();
    }
    if(with_light) {
      delay(250);
      digitalWrite(led, LOW);
      delay(250);
    }
  }
  
}


void 
sendKeys(byte * keys, unsigned int max_offset) {
  unsigned int idx;

  if(debug == true) {  
    char s[32];
    snprintf(s, 32, "[%x %x %x %x %x %x]", 
      keys[0], keys[1], keys[2], keys[3], keys[4], keys[5]);
    Serial.println(s);
  }
  
  KeyReport report_down = { 0 };
  KeyReport report_up = { 0 };
  for(idx = 0; idx < max_offset; idx++) {
    report_down.keys[idx] = keys[idx];
  }
  report_down.modifiers = 0;
  report_down.reserved = 1;
  Keyboard.sendReport(&report_down);
  // delay(2);
  Keyboard.sendReport(&report_up); 
  // delay(2);
  
  return;
}


void 
sendControl(unsigned int type) {
  
  KeyReport report_down = { 0 };
  KeyReport report_up = { 0 };
  report_down.keys[0] = type;
  report_down.modifiers = 0;
  report_down.reserved = 1;
  Keyboard.sendReport(&report_down);
  Keyboard.sendReport(&report_up); 
  
  return;
}


void setup() {

  // Init Hardware
  pinMode(led, OUTPUT);
  Keyboard.begin();
  PCserial.begin(38400);
  if(debug)
    Serial.begin(9600);

  delay(start_delay * 1000);
  // Flush PC serial port
  while(PCserial.available() > 0)
    PCserial.read();
  user_notify(3);
  delay(750);

}


void loop() {
  unsigned char octet;
  
  if(!PCserial.available()) {
    if(counter != -1) {
      if(debug)
        delay(1000);
      if(!counter)          // do once, when zero
        PCserial.write(1);
      delay(ack_delay);
      counter += ack_delay;
      if(counter >= timeout * 10) {
        if(offset) {
          sendKeys(keys, offset);
          offset = 0;
        }
        if(end_on_timeout)
          sendControl(0x37);  // . = end transfer
        counter = -1;
      }
    }
  } else {
    counter = 0;    
    octet = PCserial.read();
    processed_octets++;
    switch(octet) {
      // Raw Data (keyboard encoded)
      case 0x27: // '0'
      case 0x1e: // '1'
      case 0x1f: // '2'
      case 0x20: // '3'
      case 0x21: // '4'
      case 0x22: // '5'
      case 0x23: // '6'
      case 0x24: // '7'
      case 0x25: // '8'
      case 0x26: // '9'
      case 0x04: // 'a'
      case 0x05: // 'b'
      case 0x06: // 'c'
      case 0x07: // 'd'
      case 0x08: // 'e'
      case 0x09: // 'f'
      // Deduped data (keyboard encoded)
      case 0x10: // 'm': 
      case 0x11: // 'n':
      case 0x12: // 'o':
      case 0x13: // 'p':
      case 0x14: // 'q':
        keys[offset] = octet;
        offset++;
        if(offset == 6) {
          sendKeys(keys, offset);
          offset = 0;
        }
        break;
      case 0x37: // .
        if(offset)
          sendKeys(keys, offset);
        if(mode != 2) {
          sendControl(0x37); // .
          mode = 2;
        }
        offset = 0;
        break;
      case 0x36: // ,
        if(offset)
          sendKeys(keys, offset);
        if(mode != 1) {
          sendControl(0x36); // ,
          mode = 1;
        }
        offset = 0;
        break;
      case 0x2c: // space
        if(offset)
          sendKeys(keys, offset);
        if(mode != 0) {
          sendControl(0x2c); // space
          mode = 0;
        }
        offset = 0;
        break;
      default:
        offset = 0;
        break;
    }
  }
}
